local lib = LibIJA
local internal = lib.internal

-------------------------------------
-- 
-------------------------------------
-- use as Object_Name = LibIJA:Subclass(parent_Object) to inherent all lib functions
function lib:Subclass()
    local newClass = ZO_Object:Subclass()
	setmetatable( newClass, { __index = self } )
    return newClass
end

-------------------------------------
-- 
-------------------------------------
local traitIcons = {
	[0]		= "crafting_enchanting_glyphslot_empty",
	[1]		= "crafting_alchemy_trait_restorehealth",
	[2]		= "crafting_alchemy_trait_ravagehealth",
	[3]		= "crafting_alchemy_trait_restoremagicka",
	[4]		= "crafting_alchemy_trait_ravagemagicka",
	[5]		= "crafting_alchemy_trait_restorestamina",
	[6]		= "crafting_alchemy_trait_ravagestamina",
	[7]		= "crafting_alchemy_trait_increasespellresist",
	[8]		= "crafting_alchemy_trait_lowerspellresist",
	[9]		= "crafting_alchemy_trait_increasearmor",
	[10]	= "crafting_alchemy_trait_lowerarmor",
	[11]	= "crafting_alchemy_trait_increasespellpower",
	[12]	= "crafting_alchemy_trait_lowerspellpower",
	[13]	= "crafting_alchemy_trait_increaseweaponpower" ,
	[14]	= "crafting_alchemy_trait_lowerweaponpower",
	[15]	= "crafting_alchemy_trait_spellcrit",
	[16]	= "crafting_alchemy_trait_lowerspellcrit",
	[17]	= "crafting_alchemy_trait_weaponcrit",
	[18]	= "crafting_alchemy_trait_lowerweaponcrit",
	[19]	= "crafting_alchemy_trait_unstoppable",
	[20]	= "crafting_alchemy_trait_stun",
	[21]	= "crafting_alchemy_trait_detection",
	[22]	= "crafting_alchemy_trait_invisible",
	[23]	= "crafting_alchemy_trait_speed",
	[24]	= "crafting_alchemy_trait_reducespeed",
	[25]	= "crafting_poison_trait_protection",
	[26]	= "crafting_poison_trait_damage",
	[27]	= "crafting_poison_trait_hot",
	[28] 	= "crafting_poison_trait_dot",
	[29]	= "crafting_poison_trait_increasehealing",
	[30]	= "crafting_poison_trait_decreasehealing",
	[31]	= "crafting_alchemy_trait_heroism",
	[32]	= "crafting_alchemy_trait_timidity",
}

function lib:GetTraitIcon(trait)
	return "/esoui/art/icons/alchemy/" .. traitIcons[trait] .. ".dds"
end

-------------------------------------
-- global alchemy comparators
-------------------------------------
function IJA_IsAlchemyIngredient(itemData)		-- comparator
	return itemData.itemType == ITEMTYPE_POTION_BASE or
		itemData.itemType == ITEMTYPE_POISON_BASE or
		itemData.itemType == ITEMTYPE_REAGENT
end
function IJA_IsAlchemySolvent(itemData)			-- comparator
	return itemData.itemType == ITEMTYPE_POTION_BASE or
		itemData.itemType == ITEMTYPE_POISON_BASE
end
function IJA_IsAlchemyReagent(itemData)			-- comparator
	return itemData.itemType == ITEMTYPE_REAGENT
end
function IJA_IsAlchemyResultItem(itemData)		-- comparator
	return itemData.itemType == ITEMTYPE_POISON or
		itemData.itemType == ITEMTYPE_POTION
end

-------------------------------------
-- alchemy scene crafting slot functions
-------------------------------------
function lib:TryAddReagentsToCraft(reagents)
	if not lib:IsShowingAlchemyCraftingScene() then return end
	internal:TryAddReagentsToCraft(reagents)
end

--function lib:TryAddSolventToCraft(bagId, slotIndex, suppressSound, ignoreUsabilityRequirement)
function lib:TryAddSolventToCraft(solventData, suppressSound, ignoreUsabilityRequirement)
	if not lib:IsShowingAlchemyCraftingScene() then return end
	internal:TryAddSolventToCraft(solventData, suppressSound, ignoreUsabilityRequirement)
--	internal:TryAddSolventToCraft(bagId, slotIndex, suppressSound, ignoreUsabilityRequirement)
end

function lib:ClearAlchemySlots()
	if not lib:IsShowingAlchemyCraftingScene() then return end
	internal:ClearAlchemySlots()
end

-------------------------------------
-- item data functions
-------------------------------------
function lib:Contains(text, ...)
	-- returns boolean tableContains text
    return internal:Contains(text, ...)
end

function lib:GetItemData(itemId, comparator)
	-- returns itemData
	return internal:GetItemData(itemId, comparator)
end

function lib:IsItemLocked(bagId, slotIndex)
	-- returns boolean
	return internal:IsItemLocked(bagId, slotIndex)
end

function lib:GetReagentData(name)--------------------------------
    return internal.REAGENT_DATA[name]
end

function lib:GetTraitBaseName(traitId)
	return internal.POTIONTRAIT_DATA[traitId].name
end

function lib:GetTraitPoisonName(traitId)
	return internal.POTIONTRAIT_DATA[traitId].name2
end

function lib:GetTraitName(trait, resultType) --- ---------------------------- IJAEA_SOLVENT_TYPE
	if trait == 0 then return end
	local name = resultType == LibIJA_SOLVENT_TYPE_POTION and 'name' or 'name2'
	return internal.POTIONTRAIT_DATA[trait][name]
end

function lib:GetAlchemyBagAndSlots(solventId, reagents)
	-- returns solvent_bagId, solvent_slotIndex, reagent1_bagId, reagent1_lotIndex, reagent2_bagId, reagent2_slotIndex, reagent3_bagId, reagent3_slotIndex
	return internal:GetAlchemyBagAndSlots(solventId, reagents)
end

function lib:GetItemName(itemId)
	return internal:GetItemName(itemId)
end

function lib:GetProspectiveAlchemyResult()
	-- returns resultLink based on items in platform specific crafting slots
	local object = IsInGamepadPreferredMode() and GAMEPAD_ALCHEMY or ALCHEMY
	return GetAlchemyResultingItemLink(object:GetAllCraftingBagAndSlots())
end
	
function lib:GetPotLinkLevel(isWrit)---------------------
	return internal:GetPotLinkLevel(isWrit)
end

function lib:GetBaseIsPoison(resultId)
	return internal:GetBaseIsPoison(resultId)
end

function lib:GetAlchemyResultLink(resultType, traits, solventItemId)
	-- returns itemLink based on traitIds
	return internal:GetAlchemyResultLink(resultType, traits, solventItemId)
end

function lib:UpdateItemLinkLevel(itemLink)
	-- returns itemlink based on character level and crafting level
	return internal:UpdateItemLinkLevel(itemLink)
end

function lib:GetMatchingEncodedAlchemyTraits(conditionData)
	--[[
		local encodedAlchemyTraits = self.GetMatchingEncodedAlchemyTraits(conditionData)
		conditionData must contain 
		["encodedAlchemyTraits"] = 66309 -- number being the encodedAlchemyTraits required by a writ
	--]]
    return internal:GetMatchingEncodedAlchemyTraits(conditionData)
end

function lib:GetWritResultLink(conditionData)--------------
    return internal:GetWritResultLink(conditionData)
end

function lib:GetEncodedAlchemyTraits(traitIds)
	-- returns number encodedAlchemyTraits
	return internal:GetEncodedAlchemyTraits(traitIds)
end

function lib:GetTraitsFromEncodedAlchemyTraits(encodedAlchemyTraits)
	-- returns table traits
	return internal:GetTraitsFromEncodedAlchemyTraits(encodedAlchemyTraits)
end

function lib:RefreshSolvents()
	-- returns list of usable solvents
	return internal:RefreshSolvents()
end

function lib:CompareAlchemyResultLinks(sourceLink, sourceEncodedTraits)
	local linkEncodedTraits = tonumber(sourceLink:match("|H0:item:[%:%d]+:([%d]+)|h|h"))
	local traits = lib:GetTraitsFromEncodedAlchemyTraits(linkEncodedTraits)
	if linkEncodedTraits == sourceEncodedTraits then return true end
	
	local stop = false
	local step = 0
--	while (step == 100) or (linkEncodedTraits ~= sourceEncodedTraits) do
	while (step == 100) do
		traits = shuffleTable(traits)
		
		linkEncodedTraits = lib:GetEncodedAlchemyTraits(traits)
		step = step + 1
		
		if sourceEncodedTraits == linkEncodedTraits then
			return true
		end
	end
	return false
end

function lib:GetCombineReagentTraits(reagents)
	local combinedTraits, traitCounts = {}, {}

	if #reagents == 0 then return end
	for itemId, reagent in pairs(reagents) do
		for traitId, _ in pairs(reagent.traits) do
			if traitCounts[traitId] == nil then traitCounts[traitId] = 0 end
			traitCounts[traitId] = traitCounts[traitId] + 1
		end
	end
	
	local traitPrioritys = {}
	for traitId, count in pairs(traitCounts) do
		local traitData = internal.POTIONTRAIT_DATA[traitId]
		if traitCounts[traitData.oppositeId] then
			count = count - traitCounts[traitData.oppositeId] -- removes opposite from table
		end
		
		if (count > 1) then 
			combinedTraits[#combinedTraits+1] = traitId
		end
	end
	return combinedTraits
end

-------------------------------------
-- result functions
-------------------------------------
function lib:GetCurrentCraftingData()
	--[[
		Returns: solventType, solventData, recipeData, traitIds
		solventType = potion or poison
	--]]
 	return lib.currentData.type, lib.currentData.solvent, lib.currentData.recipe, lib.currentData.traitIds
end

-------------------------------------
-- move to easy alchemy?
-------------------------------------
--[[
function lib:GetMemoryTable()
     if not lib.alchemySelectionMemory then
        lib.alchemySelectionMemory = {}
    end
    return lib.alchemySelectionMemory
end
function lib:GetMemory(key)
     local memory = lib:GetMemoryTable()
    return memory[key]
end
function lib:SetMemory(key, data)
     local memory = lib:GetMemoryTable()
    memory[key] = data
end

function lib:SetSelectedTraits(t)-----------
     lib.currentData.traits = t
end
function lib:SetSelectedType(t)-------------
     lib.currentData.traits = t
end
function lib:SetSelectedRecipe(t)-----------
     lib.currentData.recipe = t
end
--]]

-------------------------------------
-- panel functions
-------------------------------------
function lib:ClearResultPanel()
	-- refreshes the recipe list with no data
	LIBIJA_RESULTPANEL:RefreshRecipeList()
end
function lib:SetResultPanelHeader(text)
	-- set the result panel header
 	LIBIJA_RESULTPANEL:SetResultPanelHeader(text)
end
function lib:RefreshAlchemyRecipeList(recipeData)
	-- refresh the recipe list with given data
	LIBIJA_RESULTPANEL:RefreshRecipeList(recipeData)
end
function lib:UpdatePanel(itemLink, solventId)
	-- updates the result name. solventId is used to update itemLink to the proper level, if different (sip, dram...)
	LIBIJA_RESULTPANEL:UpdatePanelResultName(itemLink, solventId)
end

function lib:GetAlchemyRecipeData()
	-- returns the recipe data from the recipe list
	return LIBIJA_RESULTPANEL.reagentList.list
end

-------------------------------------
-- 
-------------------------------------
internal.alchemyCraftingScenes = {
	["alchemy"] = "alchemy",
	["gamepad_alchemy_creation"] = "gamepad_alchemy_creation",
}

function lib:AddAlchemyCraftingScene(sceneName)
	--	add custom scenes to us with self:IsShowingAlchemyCraftingScene() check
	internal.alchemyCraftingScenes[sceneName] = sceneName
end

function lib:IsShowingAlchemyCraftingScene()
	-- returns boolean for if current scene is listed in internal.alchemyCraftingScenes
	local currentScene = SCENE_MANAGER:GetCurrentScene():GetName()
	return SCENE_MANAGER:IsShowing(internal.alchemyCraftingScenes[currentScene])
end

function lib:AddCallBack(callback)
	--[[
		adds function tables that are used in the "StateChange" callbacks and "SelectMode" hooks in events
		mode 1 = gamepad, 2 = keyboard
		index = is used to set a precedence index 1 will fire before index 2
		type "StateChange" or "SelectMode"
		doFunc the function to fire
		
		local StateChange = {
			['mode'] = 1,
			['index'] = 2,
			['type'] = "StateChange",
			['doFunc'] = function(oldState, newState)
				if newState == SCENE_SHOWING then
				elseif newState == SCENE_HIDDEN then
				end
				return true
			end
		}
		self:AddCallBack(StateChange)
		
	--]]
	local mode = callback.mode
	local callbackType = callback.type
	
	local tbl = lib.callbacks[mode][callbackType]
	table.insert(tbl, callback)
	
	table.sort(tbl, function(a, b)
		if a ~= nil and b ~= nil then
			return a.index < b.index 
		end
	end)
end

function lib:RegisterOnScanCompleteCallback(funct)---------------------------- remove?
	CALLBACK_MANAGER:RegisterCallback("IJA_ALCHEMYREADY", funct)
end

function lib:UnregisterOnScanCompleteCallback(funct)------------------------- remove?
	internal:UnregisterCallback("IJA_ALCHEMYREADY", funct)
end

-------------------------------------
-- 
-------------------------------------
local function Is_ExactMatch(a, b) return #a == #b end

local function equalityFunction(resultType, traitIds)
	return resultType == lib.currentData.resultType and
		lib:AreSameTraits(lib.currentData.traitIds, traitIds)
end

local function taskFunction(results)
	if results then
		return results
	end
end

function lib:IsUpdateInProgress()
	return lib.isUpdating
end

function lib:GetAlchemyRecipes(resultType, traitIds, solventData, OnScanComplete)
--[[	Returns all possible recipes containing the provided trait/s
		OnScanComplete can be nil
		
		local traits = {[1] = 1}
		
		local function OnScanComplete(recipeData)
			do stuff with recipeData
		end
		self:GetAlchemyRecipes(resultType, traits, OnScanComplete)
--]]

	if not equalityFunction(resultType, traitIds) then
		EVENT_MANAGER:UnregisterForUpdate("LibIJA_Scan_Delay")
		
		lib.currentData.type = resultType
		lib.currentData.solvent = solventData
		lib.currentData.traitIds = traitIds
			
--	if not lib.isUpdating then
		lib.isUpdating = true
		
		if OnScanComplete ~= nil then
			internal:OnAlchemyScanCompleted(OnScanComplete)
		end
--			internal:StartScan(traits, taskFunction)

		local ranOnce = false
		local function OnUpdateHandler()
			if ranOnce then
				EVENT_MANAGER:UnregisterForUpdate("LibIJA_Scan_Delay")
				lib.recipeData = {}
				IJA_ALCHEMY_RECIPE_SCAN:ScanForRecipes(traitIds, taskFunction, Is_ExactMatch)
			end
			ranOnce = true
		end
		
		local scanDelay = LibIJA_Data.scanDelay or 20
		EVENT_MANAGER:RegisterForUpdate("LibIJA_Scan_Delay", scanDelay * 10, OnUpdateHandler)
	else
		internal:FireCallbacks("IJA_OnScanComplete", lib.recipeData)
	end
end

function lib:OnAlchemyScanCompleted(OnScanComplete)
	internal:OnAlchemyScanCompleted(OnScanComplete)
end

-------------------------------------
-- 
-------------------------------------
local function containsNumber(num, ...)
     if num == nil then
        return false
    end
    if type(num) == "table" then
        return false
    end

    local keyList = {...}

    if type(keyList[1]) == "table" then
        keyList = keyList[1]
    end
    for i, key in ipairs(keyList) do
        if num == key then
            return true
        end
    end
    return false
end

function lib:AreSameTraits(oldTraits, newTraits)
 	if not oldTraits then return false end
	if #oldTraits ~= #newTraits then return false end
	for i=1, #oldTraits do
		if not containsNumber(oldTraits[i], newTraits) then
			return false
		end
	end
	return true
end

function lib:GetSkillRank(craftingType, abilityIndex)
 	local rankIndex
    local skillType, skillIndex = GetCraftingSkillLineIndices(craftingType)
    local abilityName, _, _, _, _, purchased, _, rankIndex = GetSkillAbilityInfo(skillType, skillIndex, abilityIndex)
    if (not purchased) then
        rankIndex = 0
    end
    return rankIndex
end

function lib:GetEntryTraitNames(traits, filterType)
	local output = {}
	for i=1, #traits do
		output[#output + 1] = lib:GetTraitName(traits[i], filterType)
	end
	return output
end
--[[
function lib:GetTraitCombinations()
	local traitCombinations = {}

	for k,v in pairs(LibIJA_Data.traitCombinations) do
		local combinedTraits = {}
		for _, traits in pairs(v) do
			table.insert(combinedTraits, traits)
		end
		table.insert(traitCombinations, combinedTraits)
	end
	return traitCombinations
end
--]]
-------------------------------------
-- 
-------------------------------------
-- Updates alchemy result tooltip
function lib:UpdateTooltip(...)
	internal:SetTooltipOverRide(true)
    internal:UpdateTooltip(...)
end

function lib:GetTooltipData()
	return internal:GetTooltipData()
end

function lib:SetTooltipOverRide(overRide)
	lib.useTooltip = overRide
end
lib.POTIONTRAIT_DATA = internal.POTIONTRAIT_DATA
lib.SOLVENT_DATA = internal.SOLVENT_DATA
--[[
	usage 
	self.tooltip = lib.keyboardTooltip
	CRAFTING_RESULTS:SetCraftingTooltip(self.tooltip)
--]]
--lib.keyboardTooltip = internal.keyboardTooltip

-------------------------------------
-- 
-------------------------------------
function lib:GetTraitCombinations()
	-- return the list of valid trait combinations created by the initial scan
    return LibIJA_Data.traitCombinations
end

-------------------------------------
-- Keybinds
-------------------------------------
function lib:AddTriggerKeybinds()
	LIBIJA_RESULTPANEL:AddTriggerKeybinds()
end

function lib:RemoveTriggerKeybinds()
	LIBIJA_RESULTPANEL:RemoveTriggerKeybinds()
end

function lib:RemoveKeybinds()
	LIBIJA_RESULTPANEL:RemoveKeybinds()
end

function lib:UpdateKeybindStrip()
	LIBIJA_RESULTPANEL:UpdateKeybindStrip()
end

function lib:AddKeybindButtonGroup(keybindStripDescriptor)
	LIBIJA_RESULTPANEL:AddKeybindButtonGroup(keybindStripDescriptor)
end
