local lib = LibIJA
local internal = lib.internal

local MODE_GP = 1
local MODE_KB = 2

-------------------------------------
-- 
-------------------------------------
local function OnScanComplete(recipeData)
	lib.isUpdating = false
	if recipeData ~= nil then
		LIBIJA_RESULTPANEL:RefreshRecipeList(recipeData)
		
		lib.useTooltip = true
--		internal:UpdateTooltip(internal:GetTooltipData())
		internal:UpdateTooltip()
		lib:UpdateKeybindStrip()
	end
end

internal:RegisterCallback("IJA_OnScanComplete", OnScanComplete)

function internal:InitializeHooks()
	lib.callbacks = {
		[MODE_GP] = {
			["SelectMode"]	= {},
			["StateChange"] = {},
		},
		[MODE_KB] = {
			["SelectMode"]	= {},
			["StateChange"] = {},
		},
	}
	lib.useTooltip = false
	local function shouldShowPannel(mode)
		LIBIJA_RESULTPANEL:SetHidden( LIBIJA_RESULTPANEL:IsHidden(mode))
	end
	
	local function SelectMode_KB(mode, object)
        CRAFTING_RESULTS:SetCraftingTooltip(nil)
		shouldShowPannel(mode)
		
		if IJA_ACTIVEWRITS and not IJA_ACTIVEWRITS[CRAFTING_TYPE_ALCHEMY] then
			lib:ClearResultPanel()
		end
		
		local rtrn = false
		for k, callback in pairs(lib.callbacks[MODE_KB]["SelectMode"]) do
			rtrn = callback.doFunc(mode, object)
			if rtrn then break end
		end
		if mode ~= ZO_ALCHEMY_MODE_RECIPES then 
--			object.tooltip = lib.keyboardTooltip
		end
		return rtrn
	end
	ZO_PreHook(ALCHEMY, "SetMode", function(self, mode)
		return SelectMode_KB(mode, self)
	end)
	
	local function stateChanged_KB(oldState, newState)
		if newState == SCENE_SHOWING then
		elseif newState == SCENE_SHOWN then
		elseif newState == SCENE_HIDDEN then
			ALCHEMY_SCENE:RemoveFragment(IJA_ALCHEMY_FRAGMENT)
			lib.useTooltip = false
			lib:RemoveKeybinds()
		end
		for k, callback in pairs(lib.callbacks[MODE_KB]["StateChange"]) do
			callback.doFunc(oldState, newState)
			
			if LibIJA_Data and LIBIJA_RESULTPANEL then 
				LIBIJA_RESULTPANEL.control:SetMovable(not LibIJA_Data.lock)
			end
		end
	end
	local keyboardScene = SCENE_MANAGER:GetScene("alchemy")
	keyboardScene:RegisterCallback("StateChange", function(oldState, newState)
		stateChanged_KB(oldState, newState)
	end)
	
	local function SelectMode_GP(mode)
		shouldShowPannel(mode)
		if IJA_ACTIVEWRITS and not IJA_ACTIVEWRITS[CRAFTING_TYPE_ALCHEMY] then
			lib:ClearResultPanel()
		end
		
		local rtrn = false
		for k, callback in pairs(lib.callbacks[MODE_GP]["SelectMode"]) do
			rtrn = callback.doFunc(mode)
			if rtrn and mode ~= ZO_ALCHEMY_MODE_RECIPES then
				ApplyTemplateToControl(ZO_GamepadAlchemyTopLevelSlotContainer, "IJA_GamepadAlchemyTopLevelSlotContainer")
			else
				ApplyTemplateToControl(ZO_GamepadAlchemyTopLevelSlotContainer, "ZO_GamepadCraftingIngredientBarTemplate")
			end
			if rtrn then break end
		end
		if mode ~= ZO_ALCHEMY_MODE_RECIPES then 
		end
		return rtrn
	end
	ZO_PreHook(GAMEPAD_ALCHEMY, "SelectMode", function(self)
		local data = self.modeList:GetTargetData()
		if data then
			return SelectMode_GP(data.mode)
		end
	end)

	local function stateChanged_GP(oldState, newState)
		if newState == SCENE_SHOWING then
		elseif newState == SCENE_SHOWN then
		elseif newState == SCENE_HIDDEN then
			GAMEPAD_ALCHEMY_CREATION_SCENE:RemoveFragment(IJA_ALCHEMY_FRAGMENT)
			lib.useTooltip = false
			lib:RemoveKeybinds()
		end
		for k, callback in pairs(lib.callbacks[MODE_GP]["StateChange"]) do
			callback.doFunc(oldState, newState)
		end
	end
	local gamepadScene = SCENE_MANAGER:GetScene("gamepad_alchemy_mode")
    gamepadScene:RegisterCallback("StateChange", function(oldState, newState)
		stateChanged_GP(oldState, newState)
	end)
	
	local function UpdateTooltip()
		lib.tooltip:SetHidden(true)
		if lib.useTooltip then
	--		internal:UpdateTooltip(internal:GetTooltipData())
			internal:UpdateTooltip()
		end
		return lib.useTooltip
	end
	
	local updateTooltip_KB = ALCHEMY.UpdateTooltip
	function ALCHEMY:UpdateTooltip()
		self.tooltip:SetHidden(true)
		if lib.useTooltip then
			internal:UpdateTooltip()
		else
			if self:IsCraftable() and self.mode ~= ZO_ALCHEMY_MODE_RECIPES then
				self.tooltip:SetHidden(false)
				self.tooltip:ClearLines()
				self.tooltip:SetPendingAlchemyItem(self:GetAllCraftingBagAndSlots())
			else
				self.tooltip:SetHidden(true)
			end
		end
	end
	--[[
	ZO_PreHook(ALCHEMY, 'UpdateTooltip', function()
		return UpdateTooltip()
	end)
	--]]
	ZO_PreHook(GAMEPAD_ALCHEMY, 'UpdateTooltip', function()
		return UpdateTooltip()
	end)
end
