local lib = LibIJA
local internal = lib.internal
local async = lib.internal.async

local function getAvgPrice(itemLink)
	if TamrielTradeCentre then
		local priceInfo = TamrielTradeCentrePrice:GetPriceInfo(itemLink)
		if priceInfo and priceInfo.Avg then
			return priceInfo.Avg
		end
	end
	return false
end

-------------------------------------
-- 
-------------------------------------
function internal:TryAddReagentsToCraft(reagents)
	local ignoreUsabilityRequirement = false
	
	local object = IsInGamepadPreferredMode() and GAMEPAD_ALCHEMY or ALCHEMY
	local numSlots = ZO_Alchemy_IsThirdAlchemySlotUnlocked() and 3 or 2
	for i=1, numSlots do
		local bagId 	= reagents[i] and reagents[i].bagId or nil
		local slotIndex = reagents[i] and reagents[i].slotIndex or nil
		
		local suppressSound = i~=1
		object.reagentSlots[i]:SetItem(bagId, slotIndex, suppressSound, ignoreUsabilityRequirement)
	end
	
	internal:FireCallbacks("IJA_ALCHEMYREADY")
end

--function internal:TryAddSolventToCraft(bagId, slotIndex, suppressSound, ignoreUsabilityRequirement)
function internal:TryAddSolventToCraft(solventData, suppressSound, ignoreUsabilityRequirement)
	local zo_Object = IsInGamepadPreferredMode() and GAMEPAD_ALCHEMY or ALCHEMY
	local bagId, slotIndex = ZO_Inventory_GetBagAndIndex(solventData)
	if not zo_Object.solventSlot:IsItemId(GetItemInstanceId(bagId, slotIndex)) then
		lib.currentData.solvent = solventData
		
		if not ignoreUsabilityRequirement then
			if not zo_Object.solventSlot:MeetsUsabilityRequirement() then
				return
			end
		end
		zo_Object.solventSlot:SetupItem(bagId, slotIndex)
		if zo_Object.solventSlot:HasItem() then
			if not suppressSound then
				PlaySound(zo_Object.solventSlot.placeSound)
			end
		else
			if not suppressSound then
	--			PlaySound(zo_Object.solventSlot.removeSound)
			end
		end
	end
	zo_Object.lastReagentIndexAdded = 1
end

function internal:ClearAlchemySlots()
	if not lib:IsShowingAlchemyCraftingScene() then return end
	
	local ignoreUsabilityRequirement = false
	local suppressSound = false
	
	local object = IsInGamepadPreferredMode() and GAMEPAD_ALCHEMY or ALCHEMY
	local numSlots = ZO_Alchemy_IsThirdAlchemySlotUnlocked() and 3 or 2
	for i=1, numSlots do
		object.reagentSlots[i]:SetItem(nil, nil, suppressSound, ignoreUsabilityRequirement)
	end
end

function internal:Contains(text, ...)
    if text == nil or text == "" then
        return nil
    end
    if type(text) == "table" then
        return false
    end

    local keyList = {...}
    if type(keyList[1]) == "table" then
        keyList = keyList[1]
    end
    local lowerText = string.lower(text)
    local result
    for _, key in ipairs(keyList) do
        if key and key ~= "" then
		key = "^" .. key .. "$"
            result, result2 = string.match(text, key)
            if result then
 --               return result, result2
                return true
            end
            local lowerKey = string.lower(key)
            if lowerText and lowerKey then
                result, result2 = string.match(lowerText, lowerKey)
                if result then
 --                   return result, result2
                    return true
                end
            end
        end
    end
--    return nil
    return false
end

function internal:GetItemData(itemId, comparator)
	if type(itemId) ~= 'number' then itemId = tonumber(itemId) end
	if itemId ~= nil then
		local invintoryCache = SHARED_INVENTORY:GenerateFullSlotData(comparator, BAG_VIRTUAL,BAG_BACKPACK,BAG_BANK, BAG_SUBSCRIBER_BANK)
		
		for index, data in pairs(invintoryCache)do
			if data ~= nil then
			if itemId == GetItemId(data.bagId, data.slotIndex) then
			end
				if itemId == GetItemId(data.bagId, data.slotIndex) and not lib:IsItemLocked(data.bagId, data.slotIndex) then
					local itemLink = GetItemLink(data.bagId, data.slotIndex)
					local icon, _, meetsUsageRequirement = GetItemLinkInfo(itemLink)
					
					local itemData = {
						name = data.name,
						bagId = data.bagId,
						slotIndex = data.slotIndex,
						itemType = data.itemType,
						itemLink = itemLink,
						itemId = itemId,
						icon = icon,
						sellPrice = getAvgPrice(itemLink) or data.sellPrice,
						itemFunctionalQuality = GetItemFunctionalQuality(data.bagId, data.slotIndex),
						meetsUsageRequirement = meetsUsageRequirement or false,
						stackCount = data.stackCount or 0
					}
					
					return itemData
				end
			end
		end
	end
end

function internal:IsItemLocked(bagId, slotIndex)
	local locked = IsItemPlayerLocked(bagId, slotIndex)
	
	if FCOIS and not locked and not IsInGamepadPreferredMode() then
		locked = FCOIS.IsLocked(bagId, slotIndex)
	end
	if locked then
		lib.lockedItems[GetItemId(bagId, slotIndex)] = GetItemName(bagId, slotIndex)
		ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, zo_strformat(GetString(SI_LIBIJA_ITEMLOCKED), GetItemName(bagId, slotIndex)))
	end
	return locked
end

function internal:GetAlchemyBagAndSlots(solventId, reagents)
	local solventData	= lib:GetItemData(solventId, IJA_IsAlchemySolvent)
	local reagent1Data	= lib:GetItemData(reagents[1].itemId, IJA_IsAlchemyReagent)
	local reagent2Data	= lib:GetItemData(reagents[2].itemId, IJA_IsAlchemyReagent)
	
	local bagId3, slotIndex3
	if reagents[3] then
		reagent3Data = lib:GetItemData(reagents[3].itemId, IJA_IsAlchemyReagent)
		bagId3, slotIndex3 = reagent3Data.bagId, reagent3Data.slotIndex
	end

	return solventData.bagId, solventData.slotIndex, reagent1Data.bagId, reagent1Data.slotIndex, reagent2Data.bagId, reagent2Data.slotIndex, bagId3, slotIndex3
end

function internal:GetItemName(itemId)
	local linkFormat = "|H0:item:<<1>>:30:1:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h"
	local itemLink = zo_strformat(linkFormat, itemId)
	return GetItemLinkName(itemLink)
end

function internal:GetPotLinkLevel(isWrit)
	local CP160 = "366:50"  -- truly superb
    local CP150 = "308:50"
    local CP100 = "134:50"
    local CP50  = "129:50"
    local CP10  = "125:50"
    local LV40  = "30:40"
    local LV30  = "30:30"
    local LV20  = "30:20"
    local LV10  = "30:10"
    local LV3   = "30:3"

    local skillType, skillIndex = GetCraftingSkillLineIndices(CRAFTING_TYPE_ALCHEMY)
    local abilityName, _, _, _, _, purchased, _, rankIndex = GetSkillAbilityInfo(skillType, skillIndex, 1)
    if (not purchased) then
        rankIndex = 0
    end
	
	local potLevel = select(rankIndex, {LV3, LV10}, LV20, LV30, LV40, CP10, CP50, CP100, CP150, CP160) or LV3
	
	if type(potLevel) == "table" then	-- for alchemy level 1
		if isWrit then
			potLevel = potLevel[1]
		else
			local playerLevel = GetUnitLevel("player")
			potLevel = playerLevel <10 and potLevel[1] or potLevel[2]
		end
	end
	return potLevel
end

function internal:UpdateItemLinkLevel(itemLink)
	local formatText = "|H0:item:<<1>>:<<2>>:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:<<3>>|h|h"
	local potLevel = internal:GetPotLinkLevel()
    local itemId, potionData = itemLink:match('|H0:item:([%d]+):[%d:]+:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:([%d]+)|h|h')

	return zo_strformat(formatText, itemId, potLevel, potionData)
end

function internal:GetBaseIsPoison(resultId)
	for k, data in pairs(internal.POTIONTRAIT_DATA) do
		if data.potionBaseId == resultId then
			return false
		end
		if data.poisonBaseId == resultId then
			return true
		end
	end
end

function internal:GetAlchemyResultLink(resultType, traitIds, solventId)
    local formatText = "|H0:item:<<1>>:<<2>>:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:<<3>>|h|h"
	solventLevel ={
		[3]		= "30:1",		-- Solvent Proficiency 1
		[10]	= "30:10",		-- Solvent Proficiency 1
		[20]	= "30:20",		-- Solvent Proficiency 2
		[30] 	= "30:30",		-- Solvent Proficiency 3
		[40] 	= "30:40",		-- Solvent Proficiency 4
		[51] 	= "125:50",		-- Solvent Proficiency 5
		[55] 	= "129:50",		-- Solvent Proficiency 6
		[60] 	= "134:50",		-- Solvent Proficiency 7
		[65] 	= "308:50",		-- Solvent Proficiency 8
	}
	
	local potLevel = solventLevel[internal.SOLVENT_DATA[solventId].level]
	
	local encodedAlchemyTraits = internal:GetEncodedAlchemyTraits(traitIds)
	local baseId = 0
	
	if resultType == 1 then
		baseId = internal.POTIONTRAIT_DATA[traitIds[1]].potionBaseId
	else
		baseId = internal.POTIONTRAIT_DATA[traitIds[1]].poisonBaseId
	end
	
	return zo_strformat(formatText, baseId, potLevel, encodedAlchemyTraits)
end

local function getReagentId(bagId, slotIndex)
	local itemId = GetItemId(bagId, slotIndex)
	return itemId > 0 and itemId or nil
end

function internal:GetAlchemyResultingItemLink()
	local zo_Object = IsInGamepadPreferredMode() and GAMEPAD_ALCHEMY or ALCHEMY
	local solventBagId, solventSlotIndex, reacgent1BagId, reacgent1Slot, reacgent2BagId, reacgent2Slot, reacgent3BagId, reacgent3Slot = zo_Object:GetAllCraftingBagAndSlots()
	local itemLink, prospectiveAlchemyResult = GetAlchemyResultingItemLink(solventBagId, solventSlotIndex, reacgent1BagId, reacgent1Slot, reacgent2BagId, reacgent2Slot, reacgent3BagId, reacgent3Slot)

	if itemLink == '' and solventSlotIndex and reacgent1Slot and reacgent2Slot then
		-- used to bypass zos function that requires knowledge
		local solventId		= GetItemId(solventBagId, solventSlotIndex)
		local reagentId_1	= getReagentId(reacgent1BagId, reacgent1Slot)
		local reagentId_2	= getReagentId(reacgent2BagId, reacgent2Slot)
		local reagentId_3	= getReagentId(reacgent3BagId, reacgent3Slot)
		
		local reagents = {internal.REAGENT_DATA[reagentId_1], internal.REAGENT_DATA[reagentId_2], internal.REAGENT_DATA[reagentId_3]}
		local traitIds = lib:GetCombineReagentTraits(reagents)
		if #traitIds > 0 then
			local resultType = internal.SOLVENT_DATA[solventId].isPoison and 2 or 1

			itemLink = internal:GetAlchemyResultLink(resultType, traitIds, solventId)
		end
	end
	
	return itemLink, prospectiveAlchemyResult
end

function internal:GetEncodedAlchemyTraits(traitIds)
 	local encodedAlchemyTraits = 0
	if #traitIds == 1 then
		encodedAlchemyTraits = traitIds[1]
	else
		for i=1, #traitIds do
			encodedAlchemyTraits = encodedAlchemyTraits * 256 + traitIds[i]
		end
	end
	return encodedAlchemyTraits
end

function internal:GetTraitsFromEncodedAlchemyTraits(encodedAlchemyTraits)
	if encodedAlchemyTraits >= NUM_ALCHEMY_TRAITS then return encodedAlchemyTraits end
	local trait1 = math.floor(encodedAlchemyTraits / 65536) % 256
	local trait2 = math.floor(encodedAlchemyTraits / 256) % 256
	local trait3 = encodedAlchemyTraits % 256
	trait2 = trait2 > 0 and trait2 or nil
	trait3 = trait3 > 0 and trait3 or nil
	
	return {trait1, trait2, trait3}
end

function internal:GetTooltipData()
	return lib.currentData.type, lib.currentData.traitIds, lib.currentData.solvent.itemId
end

function internal:SetTooltipOverRide(overRide)
	lib.useTooltip = overRide
	
	if not lib:IsShowingAlchemyCraftingScene() then return end
	if overRide then
		local ranOnce = false
		local function OnUpdateHandler()
			if ranOnce then
				EVENT_MANAGER:UnregisterForUpdate("LibIJA_HideAchemyTooltip")
	--			ALCHEMY.tooltip:SetHidden(true)
			end
			ranOnce = true
		end
	
		EVENT_MANAGER:RegisterForUpdate("LibIJA_HideAchemyTooltip", 1, OnUpdateHandler)
	end
end

function internal:GetAvgPrice(itemLink)
	if TamrielTradeCentre then
		local priceInfo = TamrielTradeCentrePrice:GetPriceInfo(itemLink)
		if priceInfo and priceInfo.Avg then
			return priceInfo.Avg
		end
	else
		return select(2, GetItemLinkInfo(itemLink))
	end
end

-------------------------------------
-- 
-------------------------------------
local function getSolventCraftingLevel(lvl)
 	solventLevel ={
		[3]		= 1,
		[10]	= 1,
		[20]	= 2,
		[30] 	= 3,
		[40] 	= 4,
		[51] 	= 5,
		[55] 	= 6,
		[60] 	= 7,
		[65] 	= 8,
	}
	return solventLevel[lvl]
end

function internal:RefreshSolvents()
 	local solvents = {}
	local itemFormat = "|H0:item:<<1>>:30:1:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h"

	local sloventIds = {
		[1] = {[1] = "883", 	[2] = "1187", 	[3] = "4570", 	[4] = "23265", 	[5] = "23266", 	[6] = "23267", 	[7] = "23268", 	[8] = "64500", 	[9] = "64501"},
		[2] = {[1] = "75357",	[2] = "75358", 	[3] = "75359", 	[4] = "75360", 	[5] = "75361", 	[6] = "75362", 	[7] = "75363", 	[8] = "75364", 	[9] = "75365"}
	}

	for i=1, 2 do
		local data = {}
		for k,itemId in ipairs(sloventIds[i]) do
			local solventLink = zo_strformat(itemFormat, itemId)
			local solventData = lib:GetItemData(itemId, IJA_IsAlchemySolvent)
			if solventData and solventData.meetsUsageRequirement and solventData.stackCount > 0 then
				solventData.solventType = i
				
				solventData.craftingLevel = getSolventCraftingLevel(lib.internal.SOLVENT_DATA[tonumber(itemId)].level)
				
				solventData.filterType = IJA_ALCHEMY_FILTER_SOLVENT
				data[k] = solventData
			end
		end
		solvents[i] = data
	end
	return solvents
end

function internal:DoesTraitListNeedUpdating(hidden)
	if hidden then
		local function printUpdatedTrait(traitId)
			d( zo_strformat(GetString(SI_LIBIJA_ALCHEMYSCAN_TRAIT_UPDATED), lib:GetTraitBaseName(traitId)))
		end
		local function updateTrait(traitData)
			traitData.labUse = true
			printUpdatedTrait(traitData.trait)
		end
		
		local traits = lib.currentData.traitIds
		local trait_1 = traits[1]
		
		if #traits > 1 then
			local traitCombinations = LibIJA_Data.traitCombinations[trait_1]
			for i=1, # traitCombinations do
				local trait_2 = traitCombinations[i]
				if #traits == 3 then
					for n=1, #trait_2 do
						local trait_3 = trait_2[n]
						if trait_3.trait == traits[3] then
							updateTrait(trait_3)
						end
					end
				else
					if trait_2.trait == traits[2] then
						updateTrait(trait_2)
					end
				end
			end
		end
	end
end

-------------------------------------
function internal:GetItemLinkFromItemId(itemId)
	 local formatText = "|H0:item:<<1>>:30:1:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h"
	return zo_strformat(formatText, itemId)
end

-------------------------------------
-- Builds the list used for trait selection
-------------------------------------
local function buildSortedTable(trait_1, input)
	local output = {}
	table.sort(input, function(a, b)
		local labA = a.labUse and 1 or 0
		local labB = b.labUse and 1 or 0
--[[
		if labA ~= labB then 
			return #a.traits < #b.traits
		end
		--]]
		return labA < labB
	end)

	for n=1, #input do
		local traits = {}
		for k, trait in pairs(input[n].traits) do
			if trait ~= trait_1 then
				table.insert(traits, trait)
				
				if  trait_1 == 1 then
					if trait == 29 then
		--				d( input[n])
					end
				end
			
			end
		end
		
		if #traits > 0 then
			local trait_2 = traits[1]
			if trait_2 ~= nil then
			
				if not output[trait_2] then output[trait_2] = {labUse = input[n].labUse} end
				local trait_3 = traits[2]
				
				if trait_3 ~= nil then
					if not output[trait_2][trait_3] then output[trait_2][trait_3] = {labUse = input[n].labUse} end

					if not output[trait_3] then output[trait_3] = {labUse = input[n].labUse} end
					if not output[trait_3][trait_2] then output[trait_3][trait_2] = {labUse = input[n].labUse} end

				else
--	if trait_2 == 12 then d( '----', traits, input[n].labUse) end 
					if not output[trait_2][0] then output[trait_2][0] = {labUse = input[n].labUse} end
				end
--				output[trait_2].labUse = output[trait_2][0].labUse
			end
		else
			if not output[0] then output[0] = {labUse = input[n].labUse} end
			if not output[0][0] then output[0][0] = {labUse = input[n].labUse} end
		end
	end
	
	return output
end	

local function buildTraitTable(trait_1, input)
	local output = {trait = trait_1}
	for k,v in pairs(buildSortedTable(trait_1, input)) do
		local key = #output + 1
		if not output[key] then output[key] = {trait = k, labUse = v.labUse } end
		
		if type(v) == 'table' then
			for r, b in pairs(v) do
				if r ~= "labUse" then
					local key2 = #output[key] + 1
					if not output[key][key2] then output[key][key2] = {trait = r, labUse = b.labUse } end
				end
			end
			table.sort(output[key], function(a, b) return a.trait < b.trait end)
--			output[key].labUse = output[key][1].labUse
		end
	end
	table.sort(output, function(a, b) return a.trait < b.trait end)
--	output.labUse = output[1].labUse
	return output
end

function internal:BuildTraitCombinations()
	local function taskFunction(results, labUse)
		if results == nil then return end
		local output = {traits = {}}
		for i=1, #results.traits do
			output.traits[i] = results.traits[i]
		end
		output["labUse"] = labUse

		return output
	end

	local traitCombinations = {}
	internal.callingFunction = 'BuildTraitCombinations'
	
	d( GetString(SI_LIBIJA_ALCHEMYSCAN_START))
	local sort = LibIJA_Data.priorityBy	--	save sort for later
	LibIJA_Data.priorityBy = IJA_REAGENT_PRIORITY_BY_ID
	
	internal.build:For (1, NUM_ALCHEMY_TRAITS):Do(function(trait_1)
--	internal.build:For (1, 1):Do(function(trait_1)
		internal.recipeData = {}
		local trait = {[1] = trait_1}
		d(zo_strformat(GetString(SI_LIBIJA_REAGENT_PRIORITY_PROCESSING), trait_1, NUM_ALCHEMY_TRAITS, lib:GetTraitBaseName(trait_1)))
		
		internal.build:Call(function()
			IJA_ALCHEMY_RECIPE_SCAN:ScanForRecipes(trait, taskFunction)
		end):Then(function()
			traitCombinations[trait_1] = buildTraitTable(trait_1, internal.recipeData)
		end)
		
	end):Finally(function()
		d( GetString(SI_LIBIJA_ALCHEMYSCAN_COMPLETE))
		LibIJA_Data.priorityBy = sort	--	reset sort
		LibIJA_Data.traitCombinations = traitCombinations
		LibIJA_Data = LibIJA_Data
	end)
end

-------------------------------------
-- 
-------------------------------------
local function Is_ExactMatch(a, b) return #a == #b end

function internal:StartScan(traits, taskFunction)
	EVENT_MANAGER:UnregisterForUpdate("LibIJA_Scan_Delay")
	local ranOnce = false
	local function OnUpdateHandler()
		if ranOnce then
			EVENT_MANAGER:UnregisterForUpdate("LibIJA_Scan_Delay")
	--		lib:ClearAlchemySlots()
		
			internal.recipeData = {}
			internal.callingFunction = "GetAlchemyRecipes"
			IJA_ALCHEMY_RECIPE_SCAN:ScanForRecipes(traits, taskFunction, Is_ExactMatch)
		end
		ranOnce = true
	end
	
	local scanDelay = LibIJA_Data.scanDelay or 20
	EVENT_MANAGER:RegisterForUpdate("LibIJA_Scan_Delay", scanDelay * 10, OnUpdateHandler)
end

function internal:OnAlchemyScanCompleted(func)
	local function OnScanComplete(...)
		internal:UnregisterCallback("IJA_ALCHEMYREADY", OnScanComplete)
		
		if func ~= nil then
			func(...)
		end
	end
	internal:RegisterCallback("IJA_ALCHEMYREADY", OnScanComplete)
end

--[[
		local hexString = "0x%i0000"
		hexString = string.format(hexString, traitIds[1])
		encodedAlchemyTraits = tonumber(hexString)
--]]