local lib = LibIJA
local internal = lib.internal
internal.resultPanel = {}

IJA_ALCHEMY_FILTER_SOLVENT = 2

-------------------------------------
-- Reagent list LibIJA.internal.resultPanel
-------------------------------------
local function colorToHex(num)
    test = math.floor(num / 255 * 100) 
    local dec = test * .01
    print(test, dec)
    local strng = string.format("%x", math.floor(dec * 255))
    
    if string.len(strng) == 1 then
		return num >= 12 and tostring(tonumber(strng,16)) or '0' .. strng
    end
    return strng
end

local function rgbToHEX(rgb)
    return colorToHex(rgb.r) .. colorToHex(rgb.g) .. colorToHex(rgb.b)
end

local function percentToRGB(percent)
    if percent == 100 then
        percent = 99
    end
    
    local r, g, b

    if (percent > 50) then
        -- green to yellow
        r = math.floor(255 * ((50 - percent % 50) / 50))
        g = 240
    else
       -- yellow to red
        r = 230
        g = math.floor(255 * (percent / 50))
    end
    b = 100
	return {r=r, g=g, b=b}
end

function getRgbColorLabel(stackCount)
	local max = math.min(stackCount, 200)
	local RGB = percentToRGB(math.floor(max / 200 * 100))
--	print( 'RGB', RGB)
    local HEX = rgbToHEX(RGB)
	return HEX, stackCount
end

local function getIngredientLabel(itemData, numNeeded)
 	local itemListName = ''
	
	if itemData.stackCount ~= nil then
		local stackLable = zo_strformat("|c<<1>><<2>>|r", getRgbColorLabel(itemData.stackCount))
		if itemData.stackCount >= numNeeded then
			itemListName = GetItemLink(itemData.bagId, itemData.slotIndex)
			return itemListName, stackLable, true	--	hasEnough == true
		else
			itemListName = zo_strformat("|c<<1>><<2>>|r","FF4040",itemData.name)
			return itemListName, stackLable, false	--	hasEnough == false
		end
	end
end

local LibIJA_ResultPanel = ZO_InitializingObject:Subclass()
function LibIJA_ResultPanel:Initialize(control)
	local template = LIB_IJA_AlchemyResultPanel_ListTemplate
	local resultPanel = control:GetNamedChild("ResultPanel")
	
	resultPanel:SetHidden(false)
	resultPanel:SetMouseEnabled(true)
	resultPanel:SetHandler("OnMoveStop", function(control) self:OnMoveStop(control) end)
	
	resultPanel.listControl:GetNamedChild("Scroll"):SetFadeGradient(1, 1, 0, 150)
	resultPanel.listControl:GetNamedChild("Scroll"):SetFadeGradient(2, -1, 0, 150)
	
	if TamrielTradeCentre then
		resultPanel.costLable:SetText(GetString(SI_REPAIR_SORT_TYPE_COST) .. ": per TTC value in reagents:")
	else
		resultPanel.costLable:SetText(GetString(SI_REPAIR_SORT_TYPE_COST) .. ": per sell value in reagents:")
	end

	IJA_RESULTPANEL_FRAGMENT = ZO_FadeSceneFragment:New(resultPanel)
	
	self.control = resultPanel
	
	local function OnGamepadPreferredModeChanged()
		local gamepadMode = IsInGamepadPreferredMode()
		
		if lib.tooltip then lib.tooltip:SetHidden(true) end
		
		if gamepadMode then
			lib.tooltip = GAMEPAD_ALCHEMY.tooltip
			if GAMEPAD_ALCHEMY_CRAFT_SCENE then
				GAMEPAD_ALCHEMY_CRAFT_SCENE:AddFragment(IJA_RESULTPANEL_FRAGMENT)
			end
		else
			lib.tooltip = lib.keyboardTooltip
			ALCHEMY_SCENE:AddFragment(IJA_RESULTPANEL_FRAGMENT)
		end
		
		self:ApplyVisualStyle(gamepadMode)
	end
	ZO_PlatformStyle:New(OnGamepadPreferredModeChanged)
	
	internal:RegisterCallback("ApplyVisualStyle", OnGamepadPreferredModeChanged)
	
	self:InitalizeReagentList()
end

local anchor = ZO_Anchor:New()

function LibIJA_ResultPanel:OnMoveStop(control)
	anchor:SetFromControlAnchor(control)
	LibIJA_Data.resultPanelAnchor = anchor
end

function LibIJA_ResultPanel:ApplyVisualStyle(gamepadMode)
	ApplyTemplateToControl(self.control, ZO_GetPlatformTemplate("LIB_IJA_AlchemyResultPanel"))
		
	if gamepadMode then
		self.control:SetMovable(false)
	else
		if LibIJA_Data then 
			self.control:SetMovable(not LibIJA_Data.lock)
		end
		
		local savedAnchor = LibIJA_Data.resultPanelAnchor		
		
		if savedAnchor ~= nil  then
			anchor:ResetToAnchor(savedAnchor)
			anchor:SetTarget(GuiRoot)
			anchor:Set(self.control)
		end
	end
end

-------------------------------------
-- 
-------------------------------------
function LibIJA_ResultPanel:InitalizeReagentList()
	local function EqualityFunction(leftData, rightData)
		return leftData.recipe == rightData.recipe
	end
	
	local function OnSelectedDataChangedCallback(selectedData, oldData, selectedDuringRebuild)
		self.control.variations:SetText(selectedData.variations)
		if selectedData.reagents == nil then return end
		
--		lib.currentData.solvent		= selectedData.solventSlot
		lib.currentData.recipe		= selectedData
		lib:TryAddReagentsToCraft(selectedData.reagents)

		self:UpdatePanelResultName(selectedData.resultLink, lib.currentData.solvent.itemId)
		-- selectedData.resultLink, selectedData.solventSlot.itemId, 
		lib:UpdateKeybindStrip()
		
		internal:SetTooltipOverRide(true)
		
		lib.useTooltip = true
		internal:UpdateTooltip()
	end

	local function SetupFunction(control, data, selected, selectedDuringRebuild, enabled)
		local goldIcon = zo_iconFormat("EsoUI/Art/currency/currency_gold.dds", 16, 16)
		local cost = 0
		
		for i=1, #data.reagents do
			local reagentLabel, stackLabel, hasEnough = getIngredientLabel(data.reagents[i], data.iterations)
			
			control.reagents[i]:SetHidden(false)
			control.reagents[i].icon:SetTexture(data.reagents[i].icon)
			control.reagents[i].name:SetText(reagentLabel)
			control.reagents[i].stackCount:SetText(zo_strformat("|cffffffStock |r ") .. stackLabel)
			
			if LibIJA_Data.priorityBy > 2 then
				cost = cost + data.reagents[i].sellPrice
			end
		end
		
		control.reagents[3]:SetHidden(#data.reagents < 3 )
		control:SetHeight((#data.reagents * 26) + 10)
		
		self.control:GetNamedChild("Cost"):SetHidden(LibIJA_Data.priorityBy < 3 )
		if LibIJA_Data.priorityBy > 2  then
			self.control.sellPrice:SetText(cost .. ' ' .. goldIcon)
		end
		local height = LibIJA_Data.priorityBy < 3 and 240 or 280
		LIB_IJA_AlchemyResultPanel:SetHeight(height)
		if selected then
		end
	end
	
	local NUM_VISIBLE_ENTRIES = 1 -- default 5
	local reagentList = ZO_HorizontalScrollList:New(self.control.listControl, "LIB_IJA_AlchemyResultPanel_ListTemplate", NUM_VISIBLE_ENTRIES, SetupFunction, EqualityFunction)
	reagentList:SetOnSelectedDataChangedCallback(OnSelectedDataChangedCallback)

	self.reagentList = reagentList
	
	self:InitTriggerKeybindDescriptors()
end

function LibIJA_ResultPanel:ClearRecipeList()
 	self.reagentList:Clear()
	self.reagentList:Commit()
end

local function getMaxRecipes(recipeData)
	if LibIJA_Data.max == 0 then
		return #recipeData
	else
		return #recipeData > LibIJA_Data.max and LibIJA_Data.max or #recipeData
	end
end

function LibIJA_ResultPanel:RefreshRecipeList(recipeData)
	-- set data for the reagent horizontal scroll list
	self.reagentList:Clear()
	
	if recipeData then
		self:UpdateSolvent()
		local numRecipes = getMaxRecipes(recipeData)
		for index=1, numRecipes do
		local entryData = recipeData[index]
			entryData:AddAdditionalData(numRecipes, index)
			
			self.reagentList:AddEntry(entryData)
		end
	end
	
	self.reagentList:Commit()
	self:SetHidden(self:IsEmpty())
	
	if recipeData then
		internal:DoesTraitListNeedUpdating(self:IsHidden())
	end
	
	internal:FireCallbacks("IJA_ALCHEMYREADY", recipeData)
end

function LibIJA_ResultPanel:IsEmpty()
	return self.reagentList:GetNumItems() == 0
end

function LibIJA_ResultPanel:IsHidden(mode)
	return self:IsEmpty() or mode == ZO_ALCHEMY_MODE_RECIPES
end

function LibIJA_ResultPanel:OnShow()
	self:AddTriggerKeybinds()
end

function LibIJA_ResultPanel:OnHide()
	self:RemoveTriggerKeybinds()
end

function LibIJA_ResultPanel:SetHidden(hidden)
	self.control:SetHidden(hidden)
--	d( 'SetHidden')

	if not lib:IsShowingAlchemyCraftingScene()	then return end
	if hidden then
--	d( 'hide')
		self:OnHide()
	else
--	d( 'show')
		self:OnShow()
	end
end

-------------------------------------
-- update Alchemy result panel
-------------------------------------
function LibIJA_ResultPanel:ClearAndResetResultPanel()
 	clearCraftingSlots()
	
	self.alchemySelectionMemory = {}
	
	self.control.resultText:SetText(nil)
	self.control.solvent:SetText(nil)
--	lib:ShowAlchemyResultPanel()
	self:ClearRecipeList()
end

local function setResultText(text)
	local control = LIBIJA_RESULTPANEL.control
 	control.resultText:SetText(zo_strformat("<<1>>",text))
	control.resultText:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
	control.resultText:ClearAnchors()
	control.resultText:SetAnchor(TOPLEFT, control.divider, TOPLEFT, 0, LIB_yPos)

	control.testText:SetText(zo_strformat("<<1>>",text))
	local origHeight = control.testText:GetTextHeight(control.testText:GetText())
	local newHeight = control.resultText:GetTextHeight(control.resultText:GetText())

	if newHeight > origHeight then
		LIB_yPos = LIB_yPos + 30
	end
end

local function getAlchemyLinkData(itemLink)
 	local itemId, potData = itemLink:match('|H0:item:([%d]+):[%d:]+:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:([%d]+)|h|h')
	return tonumber(itemId), tonumber(potData)
end

local function updateResultLink(itemLink, solventId)
	local formatText = "|H0:item:<<1>>:<<2>>:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:<<3>>|h|h"
	local linkLevels = {
		[3] 	= "30:3",
		[10] 	= "30:10",
		[20] 	= "30:20",
		[30] 	= "30:30",
		[40] 	= "30:40",
		[51] 	= "125:50",
		[55] 	= "129:50",
		[60] 	= "134:50",
		[65] 	= "308:50",
	}
	
	local itemId, potionData = getAlchemyLinkData(itemLink)
	
	local index = internal.SOLVENT_DATA[solventId].level
	local potLevel = linkLevels[index]
	
	return zo_strformat(formatText, itemId, potLevel, potionData)
end

function LibIJA_ResultPanel:UpdatePanelResultName(itemLink, solventId)
 	if not lib:IsShowingAlchemyCraftingScene() then return end
	self:UpdateSolvent()
	
	local zo_Object = IsInGamepadPreferredMode() and GAMEPAD_ALCHEMY or ALCHEMY
	local resultLink = GetAlchemyResultingItemLink(zo_Object:GetAllCraftingBagAndSlots()):gsub("%|H1", "|H0")
	
	if resultLink == '' then 
		resultLink = updateResultLink(itemLink, solventId)
	end
	
	LIB_yPos = 10 -- reset
	setResultText(resultLink)
end

function LibIJA_ResultPanel:UpdateSolvent(recipeData)
 	local recipeData = lib.reagentList.list
	
	local solventData = lib.currentData.solvent
	if solventData.stackCount then
		local iterations = recipeData.iterations or 1
		local solventLabel, stackCount, hasEnough = getIngredientLabel(solventData, iterations)
		solventLabel = solventLabel .. "	 Stock: " .. stackCount

		self.control.solvent:SetText(solventLabel)
	end
end

function LibIJA_ResultPanel:SetResultPanelHeader(text)
 	self.control.header:SetText(text)
end

-------------------------------------
-- Keybinds
-------------------------------------
function LibIJA_ResultPanel:InitTriggerKeybindDescriptors()
	 self.reagentListKeybindStripDescriptors = {
		{
			--Ethereal binds show no text, the name field is used to help identify the keybind when debugging. This text does not have to be localized.
			name = "Back",
			keybind = "UI_SHORTCUT_LEFT_SHOULDER",
			ethereal = true,
			enabled = function() return self.reagentList:GetNumItems() > 0 end,
			callback = function()
				self.reagentList:MoveRight()
			 --   PlaySound(SOUNDS.GAMEPAD_PAGE_BACK)
			end,
		},
		{
			--Ethereal binds show no text, the name field is used to help identify the keybind when debugging. This text does not have to be localized.
			name = "Next",
			keybind = "UI_SHORTCUT_RIGHT_SHOULDER",
			ethereal = true,
			enabled = function() return self.reagentList:GetNumItems() > 0 end,
			callback = function()
				self.reagentList:MoveLeft()
			 --   PlaySound(SOUNDS.GAMEPAD_PAGE_FORWARD)
			end,
		},
	}
	
--	ZO_Gamepad_AddListTriggerKeybindDescriptors(self.reagentListKeybindStripDescriptors, self.reagentList:GetList())
end

function LibIJA_ResultPanel:AddTriggerKeybinds()
	KEYBIND_STRIP:AddKeybindButtonGroup(self.reagentListKeybindStripDescriptors)
	self:UpdateKeybindStrip()
end

function LibIJA_ResultPanel:RemoveTriggerKeybinds()
	KEYBIND_STRIP:RemoveKeybindButtonGroup(self.reagentListKeybindStripDescriptors)
end

function LibIJA_ResultPanel:RemoveKeybinds()
--	KEYBIND_STRIP:RemoveKeybindButtonGroup(lib.currentKeybindStripDescriptor)
	self:RemoveTriggerKeybinds()
end

function LibIJA_ResultPanel:UpdateKeybindStrip()
	KEYBIND_STRIP:UpdateKeybindButtonGroup(self.reagentListKeybindStripDescriptors)
	KEYBIND_STRIP:UpdateKeybindButtonGroup(lib.currentKeybindStripDescriptor)
end

function LibIJA_ResultPanel:AddKeybindButtonGroup(keybindStripDescriptor)
 	if not KEYBIND_STRIP:HasKeybindButtonGroup(keybindStripDescriptor) then
		if KEYBIND_STRIP:HasKeybindButtonGroup(lib.currentKeybindStripDescriptor) then
			KEYBIND_STRIP:RemoveKeybindButtonGroup(lib.currentKeybindStripDescriptor)
		end
		KEYBIND_STRIP:AddKeybindButtonGroup(keybindStripDescriptor)
		lib.currentKeybindStripDescriptor = keybindStripDescriptor
	end
	
	self:UpdateKeybindStrip()
end

-------------------------------------
-- 
-------------------------------------
function internal:InitializeResultPanel(control)
	LIBIJA_RESULTPANEL = LibIJA_ResultPanel:New(control)
end



	
--[[
LibLibIJA_ResultPanelResultPanel
 	table.insert(LibIJA_Data, saveData) -- was to get saves from both addons
	
	
	self.control.listControl:GetNamedChild("LeftArrow"):SetScale(0.5)
	self.control.listControl:GetNamedChild("RightArrow"):SetScale(0.5)
	
	self.control.listControl:GetNamedChild("LeftArrow"):SetHeight(150)
	self.control.listControl:GetNamedChild("RightArrow"):SetHeight(150)
	
	/script IJA_EasyAlchemy_AlchemyResultPanel_SlotTemplate:SetWidth(600)
	
/script LibIJA.internal.resultPanel:ApplyVisualStyle(true)
	
	self.control.listControl:GetNamedChild("LeftArrow"):SetWidth(16)
	self.control.listControl:SetScale(0.5)
	lib.internal.result.panel
--]]
