local lib = LibIJA
local internal = lib.internal
local LAM2 = LibAddonMenu2

local INSTANCE_REFERENCE = "PriorityByManual"

local itemIdList = {}
do
	for itemId, _ in pairs(internal.REAGENT_DATA) do
		table.insert(itemIdList, itemId)
	end
end

-------------------------------------
-- 
-------------------------------------

function lib:CreateMenuForPriority()
	local isTTC = TamrielTradeCentre and 1 or 2
	local priorityList = {
		{IJA_REAGENT_PRIORITY_BY_STOCK, 	GetString(SI_LIBIJA_REAGENT_PRIORITY_BY_STOCK)},
		{IJA_REAGENT_PRIORITY_BY_MANUAL, 	GetString(SI_LIBIJA_REAGENT_PRIORITY_BY_MANUAL)},
		{IJA_REAGENT_PRIORITY_BY_PRICE, 	GetString("SI_LIBIJA_REAGENT_PRIORITY_BY_PRICE", isTTC)},
		{IJA_REAGENT_PRIORITY_BY_VALUE,		GetString("SI_LIBIJA_REAGENT_PRIORITY_BY_VALUE", isTTC)},
	}

	local values = {}
	local displays = {}
	for key, priority in pairs(priorityList) do
		if priority then
			if priority[1] > 2 and (TamrielTradeCentre and TamrielTradeCentre.ItemLookUpTable) or true then
				values[#values + 1] = priority[1]
				displays[#displays + 1] = priority[2]
			else
				values[#values + 1] = priority[1]
				displays[#displays + 1] = priority[2]
			end
		end
	end

	local menu = {
		type = "dropdown",
		name = GetString(SI_LIBIJA_REAGENT_PRIORITY_BY),
		choices = displays,
		choicesValues = values,
		getFunc = function()
			return LibIJA_Data.priorityBy
		end,
		setFunc = function(value)
			LibIJA_Data.priorityBy = value

			local isManual = (value ==  IJA_REAGENT_PRIORITY_BY_MANUAL)
			GetControl(self.control, "_PriorityByManual"):SetHidden(not isManual)
			if (isManual) and (GetControl(self.control, "_PriorityByManual").open) then
				return
			elseif (not isManual) and (not GetControl(self.control, "_PriorityByManual").open) then
				return
			end
			local laben = GetControl(self.control, "_PriorityByManual").label
			laben:GetHandler("OnMouseUp")(laben, true)
		end,
		width = "full",
	}
	return menu
end

function lib:CreateMenuForManual()
	local formatText = "|H0:item:<<1>>:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h"
	local displays = {}
	local values = {}
	for i, itemId in ipairs(itemIdList) do
		local itemLink = zo_strformat(formatText, itemId)
		local icon = GetItemLinkIcon(itemLink)
		displays[#displays + 1] = zo_iconFormat(icon, 18, 18) .. itemLink
		values[#values + 1] = itemId
	end

	if (not LibIJA_Data.priorityByManual) then
		LibIJA_Data.priorityByManual = {}
		for i, itemId in ipairs(itemIdList) do
			LibIJA_Data.priorityByManual[i] = itemId
		end
	end

	local controlList = {}


	local function getShowPriceName()
		return (TamrielTradeCentre and TamrielTradeCentre.ItemLookUpTable) and zo_strformat(GetString(SI_LIBIJA_REAGENT_SHOW_PRICE_MANUAL), "TamrielTradeCentre") or
			zo_strformat(GetString(SI_LIBIJA_REAGENT_SHOW_PRICE_MANUAL), GetString(SI_MONSTERSOCIALCLASS24))
	end
	
--	if TamrielTradeCentre and TamrielTradeCentre.ItemLookUpTable then
		controlList[#controlList + 1] = {
			type = "checkbox",
--			name = zo_strformat(GetString(SI_LIBIJA_REAGENT_SHOW_PRICE_MANUAL), "TamrielTradeCentre"),
			name = getShowPriceName(),
			getFunc = function()
				return LibIJA_Data.showPriceTTC
			end,
			setFunc = function(value)
				LibIJA_Data.showPriceTTC = value
				self:RefreshMenuForManual()
				LAM2.util.RequestRefreshIfNeeded(GetControl(self.control, "_PriorityByManual"))
			end,
			width = "full",
			default = false,
		}
--	else
--		LibIJA_Data.showPriceTTC = false
--	end

	local prefix = self.control:GetName()
	for i, itemId in ipairs(itemIdList) do
		local control = {
			type = "dropdown",
			reference = prefix .. "Reagent".. i,
			choices = displays,
			choicesValues = values,
			name = tostring(i) .. ":",
			getFunc = function()
				return LibIJA_Data.priorityByManual[i]
			end,
			setFunc = function(value)
				local oldValue = LibIJA_Data.priorityByManual[i]
				for key, reagent in pairs(LibIJA_Data.priorityByManual) do
					if reagent == value then
						LibIJA_Data.priorityByManual[key] = oldValue
						break
					end
				end
				LibIJA_Data.priorityByManual[i] = value
			end,
			width = "full",
		}
		controlList[#controlList + 1] = control
	end

	local menu = {
		type = "submenu",
		name = GetString(SI_LIBIJA_REAGENT_PRIORITY_BY_MANUAL),
		reference = INSTANCE_REFERENCE,
		controls = controlList,
	}
	return menu
end

function lib:RefreshMenuForManual()
	local goldIcon = zo_iconFormat("EsoUI/Art/currency/currency_gold.dds", 16, 16)
	local formatText = "|H0:item:<<1>>:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h"
	local list = {}
	for i, itemId in ipairs(itemIdList) do
		local itemLink = zo_strformat(formatText, itemId)
		local icon = GetItemLinkIcon(itemLink)
		local txt
		if LibIJA_Data.showPriceTTC then
			local price = internal:GetAvgPrice(itemLink)
--			local price = TamrielTradeCentrePrice:GetPriceInfo(itemLink).Avg
			txt = zo_iconFormat(icon, 18, 18) .. itemLink .. " " .. string.format('%.0f', price) .. goldIcon
		else
			txt = zo_iconFormat(icon, 18, 18) .. itemLink
		end
		list[#list + 1] = {itemId, txt}
	end
	table.sort(list, function(a, b)
		return a[1] < b[1]
	end)
	
	local displays = {}
	local values = {}
	for i, entry in ipairs(list) do
		values[#values + 1] = entry[1]
		displays[#displays + 1] = entry[2]
	end

	for i=1, NUM_ALCHEMY_REAGENTS do
		local childName = "Reagent" .. i
		GetControl(self.control, childName):UpdateChoices(displays, values)
	end
end

function lib:CreateOptionsPanel(addedOptions)
	if not LAM2 then return end
	INSTANCE_REFERENCE =  self.control:GetName() .. "_PriorityByManual"
	
	if not LibIJA_Data.scanDelay then LibIJA_Data.scanDelay = 20 end
	if not LibIJA_Data.max then LibIJA_Data.max = 0 end
	
	local submenuFunction = LAMCreateControl["submenu"]
	LAMCreateControl["submenu"] = function(parent, submenuData, controlName)
		local control = submenuFunction(parent, submenuData, controlName)
		if (not control) then
			return control
		end
		if parent ~= panel then
			return control
		end
		if submenuData.reference == INSTANCE_REFERENCE then
			control:SetHidden(LibIJA_Data.priorityBy ~=  LIB_REAGENT_PRIORITY_BY_MANUAL)
		end
		
		return control
	end
	
	local dropdownFunction = LAMCreateControl["dropdown"]
	LAMCreateControl["dropdown"] = function(parent, dropdownData, controlName)
		local control = dropdownFunction(parent, dropdownData, controlName)
		if (not control) then
			return control
		end
		if parent ~= GetControl(self.control, "_PriorityByManual") then
			return control
		end
		if lib:Contains(dropdownData.reference, {"Reagent"}) then
			control.label:SetAnchor(TOPLEFT, control, TOPLEFT, 340, 0)
		end
		return control
	end

	local panelData = {
		type = "panel",
		name = self.displayName,
		displayName = self.displayName,
		author = self.author,
		version = self.version,
		registerForDefaults = true,
		registerForRefresh = true,
	}
	local panel = LAM2:RegisterAddonPanel(self.name, panelData)
	
	local optionsTable = {}
	
	local ADD_BEFORE = 1
	local ADD_AFTER = 2
	
	local commonOptions = {
		[1] = {
			type = "header",
			name = GetString(SI_LIBIJA_ALCHEMYOPTIONS),
			width = "full",
		},
		[2] = {	-- set the delay before scan will start after a scan was requested. allows selecting new data before scanning
			type = "slider",
			name = GetString(SI_LIBIJA_ALCHEMYSCAN_MAX_DELAY),
			tooltip = GetString(SI_LIBIJA_ALCHEMYSCAN_MAX_TOOLTIP),
			min = 1,
			max = 50,
			step = 1,
			getFunc = function() return LibIJA_Data.scanDelay end,
			setFunc = function(value) LibIJA_Data.scanDelay = value end,
			width = "full",
			default = function() return 20 end,
		},
		[3] = {	-- max results to enter into the recipe list
			type = "slider",
			name = GetString(SI_LIBIJA_ALCHEMYSCAN_MAX_RESULTS),
			tooltip = GetString(SI_LIBIJA_ALCHEMYSCAN_MAX_RESULTS_TOOLTIP),
			min = 0,
			max = 25,
			step = 5,
			getFunc = function() return LibIJA_Data.max end,
			setFunc = function(value) LibIJA_Data.max = value end,
			width = "full",
			default = function() return 0 end,
		},
		[4] = {	-- lock Alchemy result panel in keyboard mode
			type = "checkbox",
			name = GetString(SI_LIBIJA_WINDOW_LOCK),
			tooltip = GetString(SI_LIBIJA_WINDOW_LOCK_TOOLTIP),
			getFunc = function()
				return LibIJA_Data.lock
			end,
			setFunc = function(value)
				LibIJA_Data.lock = value
				LIBIJA_RESULTPANEL.control:SetMovable(not value)
			end,
			width = "full",
		},
		[5] = {
			type = "header",
			name = GetString(SI_LIBIJA_REAGENT_PRIORITY_HEADER),
			width = "full",
		}
	}
	
	--[[
	SI_WINDOW_LOCK
	SI_WINDOW_LOCK_TOOLTIP
	SI_ITEM_ACTION_MARK_AS_LOCKED
	"Lock Window", -- SI_CHAT_CONFIG_LOCK
	"Unlock Window", -- SI_CHAT_CONFIG_UNLOCK
	--]]
	for k, options in pairs(addedOptions[ADD_BEFORE]) do
		table.insert(optionsTable, options)
	end
	
	for k, options in pairs(commonOptions) do
		table.insert(optionsTable, options)
	end
	table.insert(optionsTable, self:CreateMenuForPriority())
	table.insert(optionsTable, self:CreateMenuForManual())
		
	for k, options in pairs(addedOptions[ADD_AFTER]) do
		table.insert(optionsTable, options)
	end
	
	LAM2:RegisterOptionControls(self.name, optionsTable)

	CALLBACK_MANAGER:RegisterCallback("LAM-PanelControlsCreated", function(newPanel)
		if (newPanel == panel) then
			if LibIJA_Data.showPriceTTC then
				self:RefreshMenuForManual()
				LAM2.util.RequestRefreshIfNeeded(GetControl(self.control, "_PriorityByManual"))
			end
			
			GetControl(self.control, "_PriorityByManual"):SetHidden(
				LibIJA_Data.priorityBy ~=  IJA_REAGENT_PRIORITY_BY_MANUAL
			)
		end
	end)
end
