local LIB_IDENTIFIER, LIB_VERSION = "LibIJA", 08

assert(not _G[LIB_IDENTIFIER], LIB_IDENTIFIER .. " is already loaded")

local lib = ZO_CallbackObject:Subclass()
_G[LIB_IDENTIFIER] = lib
lib.name = libName
lib.version = LIB_VERSION
lib.isAlchemyAutoCraft = false

local savedVarsVERSION = 2

--Global

local async = LibAsync

local CBM = CALLBACK_MANAGER
lib.internal = CBM
lib.internal.data = {}
lib.internal.recipeData = {}
lib.internal.callingFunction = ''

lib.internal.async = async
lib.internal.build = async:Create("build")
lib.internal.scan = async:Create("alchemyScan")

LibIJA_COLOR_GREEN = "00e673"

LibIJA_SOLVENT_TYPE_POTION = 1
LibIJA_SOLVENT_TYPE_POISON = 2

IJA_REAGENT_PRIORITY_BY_ID = 0
IJA_REAGENT_PRIORITY_BY_STOCK = 1
IJA_REAGENT_PRIORITY_BY_MANUAL = 2
IJA_REAGENT_PRIORITY_BY_VALUE = 3
IJA_REAGENT_PRIORITY_BY_PRICE = 4
LIB_yPos = 10

local internal = lib.internal

local defaults = {
	displayName = "|cFF00FFIsJusta|r |cffffffAlchemy Library|r",
	name = "lib_IsJustaAlchemy",
	version = "1.2.4",
	priorityBy = IJA_REAGENT_PRIORITY_BY_STOCK,
	showPriceTTC = false,
	scanDelay = 500,
	max = 0,
	sVersion = 2,
}
internal.defaultSettings = defaults

-- make a temporary copy until the real settings are available to avoid errors when they are accessed
internal.data = ZO_ShallowTableCopy(internal.defaultSettings)

local function resetSV()
    if LibIJA_Data and savedDataVersionsMatch() then
		if LibIJA_Data.sVersion == nil then
			return true
		else
			return LibIJA_Data.sVersion ~= defaults.sVersion
		end
	else
		return true
	end
	
end

local function initSaveData()
    if resetSV() then
        LibIJA_Data = internal.data
    else
        local tempSettings = ZO_ShallowTableCopy(internal.data)
        internal.data = LibIJA_Data

        -- upgrade settings
        for key, value in pairs(tempSettings) do
            if(internal.data[key] == nil) then
                internal.data[key] = value
            end
        end

        for key in pairs(internal.data) do
            if(tempSettings[key] == nil) then
                internal.data[key] = nil
            end
        end

        internal.data.version = tempSettings.version
    end
end

function lib.internal:Initialize(control)
	LIB_IJA_Alchemy:SetDimensions(420, 280)
	LIB_IJA_Alchemy:SetAnchor(TOP, GuiRoot, nil, TOP, 0, 0)
	LIB_IJA_Alchemy:SetHidden(true)
	
	lib.displayName = defaults.displayName
	
	IJA_ALCHEMY_FRAGMENT = ZO_SimpleSceneFragment:New(control)
	IJA_ALCHEMY_FRAGMENT:SetHideOnSceneHidden(true)

	initSaveData()

	lib.currentKeybindStripDescriptor = {}
	
	lib.currentData = {
		['traitIds'] = {[1] = 1},
		['type'] = 1,
		['solvent'] = {},
		['reagents'] = {},
	}
	
	lib.lockedItems = {}
	
	internal:InitializeHooks()
	internal:InitializeResultPanel(control)
	lib.reagentList = LIBIJA_RESULTPANEL.reagentList
	
	local function onCommandEntered(args)
		if args == 'rebuild' or args == 'r' then
			-- purge and rebuild the lists
			internal:BuildTraitCombinations(taskFunction)
			ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, GetString(SI_OPTIONS_RESET))
		end
	end
	
	LIB_IJA_Alchemy:RegisterForEvent(EVENT_PLAYER_ACTIVATED, function(eventCode, ...) lib.internal:OnPlayerActivated(...) end)
	SLASH_COMMANDS["/libija"] = onCommandEntered

	lib.isInitialized = true
end

local lastTime = 0
local function getTimeElapsed(lastTime)
	local timeStamp = lastTime
	local elapsedTime = GetTimeStamp() - timeStamp
	return elapsedTime
end

function internal:OnPlayerActivated()
	LIB_IJA_Alchemy:UnregisterForEvent(EVENT_PLAYER_ACTIVATED)
	if not TamrielTradeCentre then
--		d(zo_strformat(lib.displayName),GetString(SI_LIBIJA_TTC_INSTALL))
	else
		local totalSecPerDay = 24 * 60 * 60
		local elapsedTime = GetTimeStamp() - TamrielTradeCentrePrice.PriceTable.TimeStamp
		local elapsedDays = elapsedTime/totalSecPerDay
		if (elapsedDays > 3) then
			d(zo_strformat(lib.displayName),GetString(SI_LIBIJA_TTC_UPDATE))
		end
	end

	if (not LibIJA_Data.traitCombinations) or (#LibIJA_Data.traitCombinations ~= NUM_ALCHEMY_TRAITS) then
		internal:BuildTraitCombinations()
	end
	
	ApplyTemplateToControl(GAMEPAD_ALCHEMY.toolTip, "IJA_TooltipBorder")
	internal:FireCallbacks("ApplyVisualStyle")
end


function LIB_IJA_Alchemy_Initialize(control)
	lib.internal:Initialize(LIB_IJA_Alchemy)
end

--[[
/script d(#LibIJA_Data.traitCombinations)
/script d(#LibIJA_Data.traitCombinations ~= NUM_ALCHEMY_TRAITS)
/script d(not LibIJA_Data.traitCombinations)
/script d( LibIJA_Data.sVersion)
/script d()

--]]